/*
==========================================================================
	TESTDICE.CPP
		Test suite for Dice library.
	Copyright (c) 2001-2007 by Daniel R. Collins. All rights reserved.
  May be distributed under the terms of GPL version 2 (see Copying.txt).
==========================================================================
*/

#include <assert.h>
#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "dice.h"


/*
----------------------------------------------------------------
	Roll test computer.
----------------------------------------------------------------
*/

class C_RollTester
	{
	public:
		void start (void);
		void add   (int roll);
		void end   (int expectedMin, int expectedMax);
		
	private:
		int m_rollMin, m_rollMax;
	};


// Initialize test run

void C_RollTester::start (void)
	{
	m_rollMin = INT_MAX;
	m_rollMax = INT_MIN;
	}


// Add a single roll data point

void C_RollTester::add (int roll)
	{
	if (roll < m_rollMin) m_rollMin = roll;
	if (roll > m_rollMax) m_rollMax = roll;
	}


// Check results against what we expect

void C_RollTester::end (int expectedMin, int expectedMax)
	{
	assert(m_rollMin == expectedMin);
	assert(m_rollMax == expectedMax);
	}


/*
----------------------------------------------------------------
	Main function.
----------------------------------------------------------------
*/

int main(int argc, char* argv[])
	{
  int i;
  Dice::init();
  Dice *dice = new Dice();
  C_RollTester *tester = new C_RollTester();
  char buf[16];

  //
  // Test static functions
  //

	// Test single die roll() with d12
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( Dice::roll(12) );
	tester->end(1, 12);

	// Test multi dice roll() with 3d6
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( Dice::roll(3,6) );
	tester->end(3, 18);

  //
  // Test member functions
  //

  dice->set(3, 6, 2);
  assert(!strcmp(dice->write(buf), "3d6+2"));
  assert(dice->average() == 12);
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->roll() );
	tester->end(5, 20);

  dice->set(1, 20, 7);
  assert(!strcmp(dice->write(buf), "1d20+7"));
  assert(dice->average() == 17);
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->roll() );
	tester->end(8, 27);

  dice->read("2d8+12");
  assert(!strcmp(dice->write(buf), "2d8+12"));
  assert(dice->average() == 21);
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->roll() );
	tester->end(14, 28);

  dice->read("5d6");
  assert(!strcmp(dice->write(buf), "5d6"));
  assert(dice->average() == 17);
  tester->start();
  for (i = 0; i < 50000; i++) tester->add( dice->roll() );
	tester->end(5, 30);

  dice->read("d3-5");
  assert(!strcmp(dice->write(buf), "1d3-5"));
  assert(dice->average() == -3);
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->roll() );
	tester->end(-4, -2);

  dice->read("d%");
  assert(!strcmp(dice->write(buf), "1d100"));
  assert(dice->average() == 50);
	tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->roll() );
	tester->end(1, 100);

  dice->read("1d8+5");
  assert(dice->average() == 9);
  tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->rollMin1() );
	tester->end(6, 13);

  dice->read("4d8-8");
  assert(dice->average() == 10);
  tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->rollMin1() );
	tester->end(4, 24);

  dice->read("0d6+3");
  assert(!strcmp(dice->write(buf), "3"));
  assert(dice->average() == 3);
  tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->rollMin1() );
	tester->end(3, 3);

  dice->read("");
  assert(!strcmp(dice->write(buf), "0"));
  assert(dice->average() == 0);
  tester->start();
  for (i = 0; i < 10000; i++) tester->add( dice->rollMin1() );
	tester->end(0, 0);

  // Report
  printf("TestDice suite completed successfully.\n");

  // Exit
  delete tester;
  delete dice;
	return 0;
	}

